<?php

namespace Drupal\content_synchronizer\Processors;

use Drupal\content_synchronizer\Base\BatchProcessorBase;
use Drupal\content_synchronizer\Entity\ImportEntity;

/**
 * Batch Import.
 */
class BatchImportProcessor extends BatchProcessorBase {

  /**
   * The entities to import.
   *
   * @var array
   */
  protected $entitiesToImport;

  /**
   * Launch the import of the import entity.
   *
   * @param \Drupal\content_synchronizer\Entity\ImportEntity $import
   *   The import entity.
   * @param mixed $finishCallBack
   *   The finishcallback.
   */
  public function import(ImportEntity $import, $finishCallBack = NULL) {
    $operations = $this->getBatchOperations($import, $finishCallBack);

    $batch = [
      'title'      => t('Importing entities...'),
      'operations' => $operations,
      'finished'   => get_called_class() . '::onFinishBatchProcess',
    ];

    batch_set($batch);
  }

  /**
   * Set the entities to import list.
   */
  public function setEntitiesToImport(array $entitiesToImport) {
    $this->entitiesToImport = $entitiesToImport;
  }

  /**
   * {@inheritdoc}
   */
  protected function getBatchOperations(ImportEntity $import, $finishCallBack = NULL) {
    $operations = [];

    foreach ($import->getRootsEntities() as $data) {
      $data['finishCallback'] = $finishCallBack;
      $data['importId'] = $import->id();
      $operations[] = [
        get_called_class() . '::processBatchOperation',
        [$data],
      ];
    }

    return $operations;
  }

  /**
   * Do a batch operation.
   *
   * @param array $data
   *   The data to treat.
   * @param array $context
   *   The context.
   */
  public static function processBatchOperation(array $data, array $context) {
    $importProcessor = new ImportProcessor(ImportEntity::load($data['importId']));
    $importProcessor->importEntityFromRootData($data);

    $context['results']['finishCallback'] = $data['finishCallback'];
  }

  /**
   * Callback.
   */
  public static function onFinishBatchProcess($success, $results, $operations) {

    if (array_key_exists('finishCallback', $results)) {
      self::callFinishCallback($results['finishCallback'], $results);
    }
  }

}
