<?php

namespace Drupal\content_synchronizer\Service;

use Drupal\Core\Entity\Entity;
use Drupal\content_synchronizer\Processors\ImportProcessor;

/**
 * THe entity publsher service.
 */
class EntityPublisher {

  const SERVICE_NAME = 'content_synchronizer.entity_publisher';

  /**
   * Save the entity after import.
   *
   * If the entity is revisionable, it creates a new revision.
   * If the entity is new and is a root entity, then it is unpublished.
   */
  public function saveEntity(Entity $entity, $gid = NULL) {

    // Try to create a new revision of the current entity.
    $this->saveEntityWithRevision($entity, $gid);
  }

  /**
   * Try to create a revision of the entity.
   *
   * @param \Drupal\Core\Entity\Entity $entity
   *   The entity to save.
   * @param string $gid
   *   THe gid.
   */
  protected function saveEntityWithRevision(Entity $entity, $gid) {

    if ($entity->id()) {
      try {
        $revision = clone($entity);
        $revision->setNewRevision(TRUE);
        $revision->revision_log = '[Content Synchronizer] ' . ImportProcessor::getCurrentImportProcessor()
          ->getImport()
          ->label();
        $revision->setRevisionCreationTime(time());

        $revision->save();
      }
      catch (\Exception $e) {
        $this->saveEntityWithUnpublishedStatus($entity, $gid);
      }
    }
    else {
      $this->saveEntityWithUnpublishedStatus($entity, $gid);
    }
  }

  /**
   * Try to unpublish entity if it needs to be created. Either default save.
   *
   * @param \Drupal\Core\Entity\Entity $entity
   *   The entity to save.
   * @param string $gid
   *   THe gid.
   */
  protected function saveEntityWithUnpublishedStatus(Entity $entity, $gid) {
    if ($gid && $this->isPublishable($entity)) {

      if (!$entity->id()) {
        // If entity is root and needs to be created then, do not publish it.
        if (ImportProcessor::getCurrentImportProcessor()->getImport()->isRootEntity($gid)) {
          $entity->setPublished(FALSE);
        }
        else {
          $entity->setPublished(TRUE);
        }
      }
    }

    $this->defaultSave($entity);
  }

  /**
   * Save without publish status care.
   *
   * @param \Drupal\Core\Entity\Entity $entity
   *   The entity to save.
   */
  protected function defaultSave(Entity $entity) {
    $entity->save();
  }

  /**
   * Check if the entity is publishable.
   *
   * @param \Drupal\Core\Entity\Entity $entity
   *   The entity to check.
   *
   * @return bool
   *   The publishable state.
   */
  protected function isPublishable(Entity $entity) {
    return method_exists($entity, 'setPublished');
  }

}
